' Copyright 2016 European Union.
' Licensed under the EUPL (the 'Licence');
'
' * You may not use this work except in compliance with the Licence.
' * You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
' * Unless required by applicable law or agreed to in writing,
'   software distributed under the Licence is distributed on an "AS IS" basis,
'   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
'
' See the LICENSE.txt for the specific language governing permissions and limitations.
Imports System.Collections.Generic

Public Class Configuration
	Public FilePath As String
	Public GnUfromCycle As Boolean
	Public BatchMode As Boolean
	Public ModOut As Boolean
	Public Mod1Hz As Boolean
	Public BATCHoutpath As String	'Output path for BATCH Mode
	Public BATCHoutSubD As Boolean
	Public DistCorr As Boolean
	Public LogSize As Single
	Public AirDensity As Single
	Public OpenCmd As String
	Public OpenCmdName As String
	Public FuelDens As Single
	Public CO2perFC As Single
	Public FirstRun As Boolean
	Public DeclMode As Boolean

	Private Const FormatVersion As Short = 2

	Public Sub New()
		SetDefault()
	End Sub

	Public Sub DeclInit()
		AirDensity = cDeclaration.AirDensity
		FuelDens = cDeclaration.FuelDens
		CO2perFC = cDeclaration.CO2perFC
		DistCorr = True
		GnUfromCycle = False
	End Sub

	Public Sub SetDefault()
		GnUfromCycle = True
		BatchMode = False
		ModOut = True
		Mod1Hz = False
		BATCHoutpath = sKey.JobPath
		BATCHoutSubD = False
		DistCorr = True
		LogSize = 2
		AirDensity = 1.2
		OpenCmd = "notepad"
		OpenCmdName = "Notepad"
		FuelDens = 0.835
		CO2perFC = 3.153
		FirstRun = True
		DeclMode = True
	End Sub

	Public Sub Load()
		SetDefault()

		If Not IO.File.Exists(FilePath) Then
			Exit Sub
		End If

		Dim json As New JSON
		If Not json.ReadFile(FilePath) Then
			GUImsg(tMsgID.Err, "Failed to load settings! Using default settings.")
			Exit Sub
		End If

		Try
			Dim fileVersion As Short = json.Content("Header")("FileVersion")

			If fileVersion < 2 Then
				BatchMode = (json.Content("Body")("LastMode") = 1)
			Else
				BatchMode = json.Content("Body")("LastModeBatch")
			End If

			Try
				Mod1Hz = json.Content("Body")("Mod1Hz")
			Catch
			End Try

			ModOut = json.Content("Body")("ModOut")
			DistCorr = json.Content("Body")("DistCorrection")
			GnUfromCycle = json.Content("Body")("UseGnUfromCycle")
			LogSize = json.Content("Body")("LogSize")
			BATCHoutpath = json.Content("Body")("BATCHoutpath")
			BATCHoutSubD = json.Content("Body")("BATCHoutSubD")
			AirDensity = json.Content("Body")("AirDensity")
			FuelDens = json.Content("Body")("FuelDensity")
			CO2perFC = json.Content("Body")("CO2perFC")
			OpenCmd = json.Content("Body")("OpenCmd")
			OpenCmdName = json.Content("Body")("OpenCmdName")
			FirstRun = json.Content("Body")("FirstRun")
			DeclMode = json.Content("Body")("DeclMode")
		Catch ex As Exception
			GUImsg(tMsgID.Err, "Error while loading settings!")
		End Try
	End Sub

	Public Sub Save()
		Dim json As New JSON
		Dim dic As Dictionary(Of String, Object)

		dic = New Dictionary(Of String, Object)
		dic.Add("CreatedBy", Lic.LicString & " (" & Lic.GUID & ")")
		dic.Add("Date", Now.ToString)
		dic.Add("AppVersion", VECTOvers)
		dic.Add("FileVersion", FormatVersion)
		json.Content.Add("Header", dic)

		dic = New Dictionary(Of String, Object)
		dic.Add("LastModeBatch", BatchMode)
		dic.Add("ModOut", ModOut)
		dic.Add("Mod1Hz", Mod1Hz)
		dic.Add("DistCorrection", DistCorr)
		dic.Add("UseGnUfromCycle", GnUfromCycle)
		dic.Add("LogSize", LogSize)
		dic.Add("BATCHoutpath", BATCHoutpath)
		dic.Add("BATCHoutSubD", BATCHoutSubD)
		dic.Add("AirDensity", AirDensity)
		dic.Add("FuelDensity", FuelDens)
		dic.Add("CO2perFC", CO2perFC)
		dic.Add("OpenCmd", OpenCmd)
		dic.Add("OpenCmdName", OpenCmdName)
		dic.Add("FirstRun", FirstRun)
		dic.Add("DeclMode", DeclMode)
		json.Content.Add("Body", dic)

		json.WriteFile(FilePath)
	End Sub
End Class

